/**************************************************
 * Timing of the LAPACK general system solver DGESV
 **************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>


int N=2000, ZERO=0, ONE=1;
clock_t timespan;

int main(int argc, char* argv[]) {
    double *A, *x, *y, alpha, beta, s, t;
    int *ipiv, i, j, n, info;
    char transpose = 'N';

    /* Allocate memory space */

    x = (double*) malloc(N*sizeof(double));
    y = (double*) malloc(N*sizeof(double));
    A = (double*) malloc(N*N*sizeof(double));
    ipiv = (int *) malloc(N*sizeof(int));

    n = N;

    /* Initialize mat
rix  A  and vector  x  */
    
    for (j = 0; j < n; j++){
	for (i = 0; i < n; i++){
	    A[i+j*n] = (double)rand()/(double)RAND_MAX;;
	}
	x[j] = 1.0;
//	ipiv[i] = 0;
    }

    /* Multiply  y = alpha*A*x + beta*y  (alpha=1, beta=0)  */

    alpha = 1.0; beta = 0.0;
    dgemv_("N", &n, &n, &alpha, A, &n, x, &ONE, &beta, y, &ONE);

    /* Solve  A * x = y */

    timespan=clock();

    dgesv_(&n, &ONE, A, &n, ipiv, y, &n, &info);

    timespan=clock()-timespan;

    /* Check result */

    s = 0.0;
    for (i = 0; i < n; i++)
	s = s + (x[i] - y[i])*(x[i] - y[i]);
    s = sqrt(s);

    printf("Matrix order: n = %d\n", n);
/*    printf("Execution time: %d clocks (CLOCKS_PER_SEC = %d)\n\n",
      timespan,CLOCKS_PER_SEC); */
    t = timespan / (double)CLOCKS_PER_SEC;
    printf("Execution time: %g sec\n\n",t);
    printf("Residual error: s = %g\n", s);

} /* main */
